@@ -23,11 +23,12 @@
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
-
+#include <gdk/gdkkeysyms.h>
 #include "ut_endian.h"
 #include "xap_UnixApp.h"
 #include "xap_UnixFontManager.h"
 #include "xap_UnixFont.h"
+#include "xap_UnixNullGraphics.h"
 
 #include "gr_UnixGraphics.h"
 #include "gr_UnixImage.h"
@@ -45,6 +46,10 @@
 #include <X11/Xlib.h>
 #include <X11/Xft/Xft.h>
 
+#ifndef FT_ENCODING_ADOBE_CUSTOM
+#define FT_ENCODING_ADOBE_CUSTOM ft_encoding_adobe_custom
+#endif
+
 static const UT_uint32 adobeDUni[/*202*/][2] =
 	{
 		{0x0020,0x0020},
@@ -473,6 +478,8 @@ extern UT_uint32 adobeToUnicode(UT_uint32 iAdobe)
 
 extern UT_uint32 adobeDingbatsToUnicode(UT_uint32 iAdobe)
 {
+	
+#if 1
 	UT_uint32 low = adobeDUni[0][0];
 	UT_uint32 high = adobeDUni[202][0];
 	if(iAdobe < low)
@@ -499,18 +506,22 @@ extern UT_uint32 adobeDingbatsToUnicode(UT_uint32 iAdobe)
 		return iAdobe;
 	}
 	return adobeDUni[slow][1];
+#endif
 }
 
-const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
-										 const char* pszFontStyle,
-										 const char* pszFontVariant,
-										 const char* pszFontWeight,
-										 const char* pszFontStretch,
-										 const char* pszFontSize)
+const char* GR_UnixGraphics::findNearestFont(const char* pszFontFamily,
+											 const char* pszFontStyle,
+											 const char* pszFontVariant,
+											 const char* pszFontWeight,
+											 const char* pszFontStretch,
+											 const char* pszFontSize)
 {
-
-	XAP_UnixFont* pUnixFont = XAP_UnixFontManager::pFontManager->findNearestFont(pszFontFamily, pszFontStyle, pszFontVariant, pszFontWeight,
+	xxx_UT_DEBUGMSG(("Find Font findNearestFont for %s \n",pszFontFamily));
+	XAP_UnixFont* pUnixFont = XAP_UnixFontManager::pFontManager->findNearestFont(pszFontFamily, pszFontStyle,
+																				 pszFontVariant, pszFontWeight,
 																				 pszFontStretch, pszFontSize);
+	xxx_UT_DEBUGMSG(("Return Font name %s \n",pUnixFont->getName()));
+	//	UT_ASSERT(0);
 	return pUnixFont->getName();
 }
 
@@ -522,20 +533,12 @@ const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
 
 UT_uint32 				GR_UnixGraphics::s_iInstanceCount = 0;
 
-#ifndef WITH_PANGO
-GR_UnixGraphics::GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App * app)
-#else
-	GR_UnixGraphics::GR_UnixGraphics(GdkWindow * win, XAP_App * app)
-#endif
-		:
-         m_iLineWidth(tlu(1))
+GR_UnixGraphics::GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App * app):m_iLineWidth(tlu(1))
 {
 	m_pApp = app;
 	m_pWin = win;
-#ifndef WITH_PANGO
 	m_pFontManager = fontManager;
 	m_pFont = NULL;
-#endif
 	m_pSingleByteFont = NULL;
 	m_pMultiByteFont = NULL;
 	m_pFontGUI = NULL;
@@ -599,27 +602,116 @@ GR_UnixGraphics::GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontMana
 	m_bIsSymbol = false;
 	m_bIsDingbat = false;
 
-#ifndef WITH_PANGO
 	if (m_pFontManager)
 		m_pFallBackFontHandle = new XAP_UnixFontHandle(m_pFontManager->getDefaultFont(),
 													   FALLBACK_FONT_SIZE);
 	else
 		m_pFallBackFontHandle = NULL;
-#endif
+}
+
+bool GR_UnixGraphics::isDingbat(void) const
+{
+	return m_bIsDingbat;
+}
+
+
+bool GR_UnixGraphics::isSymbol(void) const
+{
+	return m_bIsSymbol;
+}
+
+GR_UnixGraphics::GR_UnixGraphics(GdkPixmap * win, XAP_UnixFontManager * fontManager, XAP_App * app, bool bUseDrawable):m_iLineWidth(tlu(1))
+{
+	m_pApp = app;
+	m_pWin = static_cast<GdkWindow *>(win);
+	m_pFontManager = fontManager;
+	m_pFont = NULL;
+	m_pSingleByteFont = NULL;
+	m_pMultiByteFont = NULL;
+	m_pFontGUI = NULL;
+	s_iInstanceCount++;
+	m_pColormap = gdk_rgb_get_colormap();
+
+
+	//
+	// Martin's attempt to make double buffering work.with xft
+	//
+	m_iXoff = 0;
+	m_iYoff = 0;
+	GdkDrawable * realDraw = static_cast<GdkDrawable *>(win);
+	if(bUseDrawable)
+	{
+		realDraw = static_cast<GdkDrawable *>(win);
+	}
+
+    m_iXoff = tlu(m_iXoff); m_iYoff = tlu(m_iYoff);
+	m_pGC = gdk_gc_new(realDraw);
+	m_pXORGC = gdk_gc_new(realDraw);
+	m_pVisual = GDK_VISUAL_XVISUAL( gdk_drawable_get_visual(realDraw));
+	m_Drawable = gdk_x11_drawable_get_xid(realDraw);
+
+	m_pXftFontL = NULL;
+	m_pXftFontD = NULL;
+	m_Colormap = GDK_COLORMAP_XCOLORMAP(m_pColormap);
+	m_pXftDraw = XftDrawCreate(GDK_DISPLAY(), m_Drawable, m_pVisual, m_Colormap);
+	gdk_gc_set_function(m_pXORGC, GDK_XOR);
+
+ 	GdkColor clrWhite;
+	clrWhite.red = clrWhite.green = clrWhite.blue = 65535;
+	gdk_colormap_alloc_color (m_pColormap, &clrWhite, FALSE, TRUE);
+	gdk_gc_set_foreground(m_pXORGC, &clrWhite);
+
+ 	GdkColor clrBlack;
+	clrBlack.red = clrBlack.green = clrBlack.blue = 0;
+	gdk_colormap_alloc_color (m_pColormap, &clrBlack, FALSE, TRUE);
+	gdk_gc_set_foreground(m_pGC, &clrBlack);
+
+	m_XftColor.color.red = clrBlack.red;
+	m_XftColor.color.green = clrBlack.green;
+	m_XftColor.color.blue = clrBlack.blue;
+	m_XftColor.color.alpha = 0xffff;
+	m_XftColor.pixel = clrBlack.pixel;
+
+	// I only want to set CAP_NOT_LAST, but the call takes all
+	// arguments (and doesn't have a default value).  Set the
+	// line attributes to not draw the last pixel.
+
+	// We force the line width to be zero because the CAP_NOT_LAST
+	// stuff does not seem to work correctly when the width is set
+	// to one.
+
+	gdk_gc_set_line_attributes(m_pGC,   0,GDK_LINE_SOLID,GDK_CAP_NOT_LAST,GDK_JOIN_MITER);
+	gdk_gc_set_line_attributes(m_pXORGC,0,GDK_LINE_SOLID,GDK_CAP_NOT_LAST,GDK_JOIN_MITER);
+
+	// Set GraphicsExposes so that XCopyArea() causes an expose on
+	// obscured regions rather than just tiling in the default background.
+	gdk_gc_set_exposures(m_pGC,1);
+	gdk_gc_set_exposures(m_pXORGC,1);
+
+	m_cs = GR_Graphics::GR_COLORSPACE_COLOR;
+	m_cursor = GR_CURSOR_INVALID;
+	setCursor(GR_CURSOR_DEFAULT);
+	m_bIsSymbol = false;
+	m_bIsDingbat = false;
+
+	if (m_pFontManager)
+		m_pFallBackFontHandle = new XAP_UnixFontHandle(m_pFontManager->getDefaultFont(),
+													   FALLBACK_FONT_SIZE);
+	else
+		m_pFallBackFontHandle = NULL;
 }
 
 GR_UnixGraphics::~GR_UnixGraphics()
 {
+	DELETEP(m_pFontGUI);
+
 	s_iInstanceCount--;
-	DELETEP (m_pFontGUI);
 
 	/* WARNING: Don't use XftDrawDestroy.  XftDrawDestroy will also destroy the drawable */
 	if (m_pXftDraw)
 		free(m_pXftDraw);
 
-#ifndef WITH_PANGO
 	delete m_pFallBackFontHandle;
-#endif
 
 	UT_VECTOR_PURGEALL(UT_Rect*, m_vSaveRect);
 
@@ -651,8 +743,12 @@ bool GR_UnixGraphics::queryProperties(GR_Graphics::Properties gp) const
 
 void GR_UnixGraphics::setZoomPercentage(UT_uint32 iZoom)
 {
-	DELETEP (m_pFontGUI);
 	GR_Graphics::setZoomPercentage (iZoom); // chain up
+
+	// If we have a font at all then fetch a device font suitable for the new
+	// zoom level.
+	if (m_pFont)
+		m_pXftFontD = m_pFont->getDeviceXftFont(iZoom);
 }
 
 static GdkCapStyle mapCapStyle ( GR_Graphics::CapStyle in )
@@ -669,17 +765,29 @@ static GdkCapStyle mapCapStyle ( GR_Graphics::CapStyle in )
     }
 }
 
-static GdkLineStyle mapLineStyle ( GdkGC* pGC, GR_Graphics::LineStyle in )
+static GdkLineStyle mapLineStyle ( GdkGC				  * pGC, 
+								   GR_Graphics::LineStyle 	in, 
+								   gint 					iWidth )
 {
+	iWidth = iWidth == 0 ? 1 : iWidth;
 	switch ( in )
     {
 		case GR_Graphics::LINE_ON_OFF_DASH :
+			{
+				gint8 dash_list[2] = { 4*iWidth, 4*iWidth };
+				gdk_gc_set_dashes(pGC, 0, dash_list, 2);
+			}
 			return GDK_LINE_ON_OFF_DASH ;
 		case GR_Graphics::LINE_DOUBLE_DASH :
+			{
+				gint8 dash_list[2] = { 4*iWidth, 4*iWidth };
+				gdk_gc_set_dashes(pGC, 0, dash_list, 2);
+			}
 			return GDK_LINE_DOUBLE_DASH ;
 		case GR_Graphics::LINE_DOTTED:
 			{
-				gint8 dash_list[2] = { 1, 2 }; // MARCM: I chose a pattern of 1 pixel on, 2 pixels off for dotted lines
+				/* strange but 1/3 ratio looks dotted */
+				gint8 dash_list[2] = { iWidth, 3*iWidth };
 				gdk_gc_set_dashes(pGC, 0, dash_list, 2);
 			}
 			return GDK_LINE_ON_OFF_DASH;
@@ -708,12 +816,13 @@ void GR_UnixGraphics::setLineProperties ( double inWidth,
 										  GR_Graphics::CapStyle inCapStyle,
 										  GR_Graphics::LineStyle inLineStyle )
 {
-	gdk_gc_set_line_attributes ( m_pGC, static_cast<gint>(tduD(inWidth)),
-								 mapLineStyle ( m_pGC, inLineStyle ),
+	gint iWidth = static_cast<gint>(tduD(inWidth));
+	gdk_gc_set_line_attributes ( m_pGC, iWidth,
+								 mapLineStyle ( m_pGC, inLineStyle, iWidth ),
 								 mapCapStyle ( inCapStyle ),
 								 mapJoinStyle ( inJoinStyle ) ) ;
-	gdk_gc_set_line_attributes ( m_pXORGC, static_cast<gint>(tduD(inWidth)),
-								 mapLineStyle ( m_pGC, inLineStyle ),
+	gdk_gc_set_line_attributes ( m_pXORGC, iWidth,
+								 mapLineStyle ( m_pXORGC, inLineStyle, iWidth ), /* this was m_pGC before */
 								 mapCapStyle ( inCapStyle ),
 								 mapJoinStyle ( inJoinStyle ) ) ;
 }
@@ -721,19 +830,27 @@ void GR_UnixGraphics::setLineProperties ( double inWidth,
 void GR_UnixGraphics::drawGlyph(UT_uint32 Char, UT_sint32 xoff, UT_sint32 yoff)
 {
 	UT_uint32 iChar = Char;
-	if(m_bIsSymbol && iChar < 255  && iChar >= 32)
+	if(isSymbol() && (iChar < 255)  && (iChar >= 32))
 	{
 		iChar = adobeToUnicode(Char);
-		xxx_UT_DEBUGMSG(("DrawGlyph 1 remapped %d to %d \n",Char,iChar));
+		xxx_UT_DEBUGMSG(("DrawGlyph 1 Symbol remapped %d to %d \n",Char,iChar));
 	}
-	if(m_bIsDingbat && iChar < 255  && iChar >= 32)
+	if(isDingbat())
 	{
-		iChar = adobeDingbatsToUnicode(Char);
-		xxx_UT_DEBUGMSG(("DrawGlyph 1 remapped %d to %d \n",Char,iChar));
+		FT_Face face = XftLockFace(m_pXftFontD);
+		//
+		// Only CUSTOM encoding gives non-zero indexes
+		FT_Select_Charmap(face,FT_ENCODING_ADOBE_CUSTOM);
+		iChar =  FT_Get_Char_Index(face,Char);
+		XftUnlockFace (m_pXftFontD);
 	}
 	
 	// FIXME ascent in wrong unit
-	XftDrawGlyphs(m_pXftDraw, &m_XftColor, m_pXftFontD, tdu(xoff + m_iXoff), tdu(m_pXftFontL->ascent * getResolution() / s_getDeviceResolution() + yoff + m_iYoff), &iChar, 1);
+
+	UT_sint32 idy = _tduY(yoff);
+	UT_sint32 idx = _tduX(xoff);
+
+	XftDrawGlyphs(m_pXftDraw, &m_XftColor, m_pXftFontD, tdu(m_iXoff) +idx, tdu(m_pXftFontL->ascent * getResolution() / s_getDeviceResolution() + m_iYoff)+idy, &iChar, 1);
 }
 
 void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
@@ -744,46 +861,52 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 	if (iLength == 0)
 		return;
 	// FIXME shouldn't need to do this - plam
-	yoff += m_pXftFontL->ascent * getResolution() / s_getDeviceResolution();
+	UT_sint32 idy = _tduY(yoff);
+	UT_sint32 idx = _tduX(xoff);
+
+	//	UT_sint32 iAscent = m_pXftFontL->ascent * getResolution() / s_getDeviceResolution();
+	UT_sint32 iAscent = getFontAscent();
+
 	if (!pCharWidths)
 	{
-		if(!m_bIsSymbol && !m_bIsDingbat)
+		if(!isSymbol() && !isDingbat())
 		{
-			XftDrawString32(m_pXftDraw, &m_XftColor, m_pXftFontD, tdu(xoff + m_iXoff), tdu(yoff + m_iYoff),
+			XftDrawString32(m_pXftDraw, &m_XftColor, m_pXftFontD, tdu(m_iXoff) + idx, tdu(iAscent + m_iYoff)+idy,
 							const_cast<XftChar32*> (pChars + iCharOffset), iLength);
 		}
-		else if(m_bIsSymbol)
+		else if(isSymbol())
 		{
 			xxx_UT_DEBUGMSG(("Doing draw symbols length %d offset %d \n",iLength,iCharOffset));
 			UT_uint32 * uChars = new UT_uint32[iLength];
 			for(UT_uint32 i = static_cast<UT_uint32>(iCharOffset); i< static_cast<UT_uint32>(iLength); i++)
 			{
 				uChars[i] = static_cast<UT_uint32>(pChars[iCharOffset + i]);
-				if(uChars[i] < 255 && uChars[i] >= 32)
+				if((uChars[i] < 255) && (uChars[i] >= 32))
 				{
 					uChars[i] = adobeToUnicode(uChars[i]);
 					xxx_UT_DEBUGMSG(("drawchars: mapped %d to %d \n",pChars[i],uChars[i]));
 				}
 			}
-			XftDrawString32(m_pXftDraw, &m_XftColor, m_pXftFontD, tdu(xoff + m_iXoff), tdu(yoff + m_iYoff),
+			XftDrawString32(m_pXftDraw, &m_XftColor, m_pXftFontD, tdu(m_iXoff) + idx, tdu(iAscent + m_iYoff)+idy,
 							const_cast<XftChar32*> (uChars), iLength);
 			delete [] uChars;
 		}
-		else if(m_bIsDingbat)
+		else if(isDingbat())
 		{
-			xxx_UT_DEBUGMSG(("Doing draw symbols length %d offset %d \n",iLength,iCharOffset));
-			UT_uint32 * uChars = new UT_uint32[iLength];
+			xxx_UT_DEBUGMSG(("Doing draw Dingbat length %d offset %d \n",iLength,iCharOffset));
+			FT_UInt * uChars = new FT_UInt[iLength];
+			FT_Face face = XftLockFace(m_pXftFontD);
+		//
+		// Only CUSTOM encoding gives non-zero indexes
+			FT_Select_Charmap(face,FT_ENCODING_ADOBE_CUSTOM);
 			for(UT_uint32 i = static_cast<UT_uint32>(iCharOffset); i< static_cast<UT_uint32>(iLength); i++)
 			{
-				uChars[i] = static_cast<UT_uint32>(pChars[iCharOffset + i]);
-				if(uChars[i] < 255 && uChars[i] >= 32)
-				{
-					uChars[i] = adobeDingbatsToUnicode(uChars[i]);
-					xxx_UT_DEBUGMSG(("drawchars: mapped %d to %d \n",pChars[i],uChars[i]));
-				}
+				uChars[i] = static_cast<FT_UInt>(FT_Get_Char_Index(face,pChars[iCharOffset + i]));
 			}
-			XftDrawString32(m_pXftDraw, &m_XftColor, m_pXftFontD, tdu(xoff + m_iXoff), tdu(yoff + m_iYoff),
-							const_cast<XftChar32*> (uChars), iLength);
+			XftUnlockFace (m_pXftFontD);
+			XftDrawGlyphs(m_pXftDraw, &m_XftColor, m_pXftFontD, 
+						  tdu(m_iXoff) +idx, tdu(iAscent + m_iYoff)+idy,
+						  uChars, iLength);
 			delete [] uChars;
 		}
 	}
@@ -792,8 +915,18 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 		UT_uint32 uChar;
 		XftCharSpec aCharSpec[256];
 		XftCharSpec* pCharSpec = aCharSpec;
-		UT_sint32 currentYoff = tdu(yoff);
+		XftGlyphSpec * pGlyphSpec = NULL;
+		FT_Face face = NULL;
 
+		UT_sint32 currentYoff = idy +tdu(iAscent);
+		if(isDingbat())
+		{
+			face = XftLockFace(m_pXftFontD);
+			//
+			// Only CUSTOM encoding gives non-zero indexes
+			FT_Select_Charmap(face,FT_ENCODING_ADOBE_CUSTOM);
+			pGlyphSpec = new XftGlyphSpec[iLength];
+		}
 		if (iLength > 256)
 			pCharSpec = new XftCharSpec[iLength];
 
@@ -801,15 +934,15 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 		for (int i = 0; i < iLength; ++i)
 		{
 			uChar = static_cast<UT_uint32>(pChars[i + iCharOffset]);
-			if(m_bIsSymbol && uChar < 255 && uChar >=32)
+			if(isSymbol() && uChar < 255 && uChar >=32)
 			{
 				pCharSpec[i].ucs4 = static_cast<FT_UInt>(adobeToUnicode(uChar));
-				xxx_UT_DEBUGMSG(("DrawGlyph 2 remapped %d to %d \n",uChar,pCharSpec[i].ucs4));
+				xxx_UT_DEBUGMSG(("DrawGlyph 2 Symbol remapped %d to %d \n",uChar,pCharSpec[i].ucs4));
 			}
-			else if(m_bIsDingbat && uChar < 255 && uChar >=32)
+			else if(isDingbat())
 			{
-				pCharSpec[i].ucs4 = static_cast<FT_UInt>(adobeDingbatsToUnicode(uChar));
-				xxx_UT_DEBUGMSG(("DrawGlyph 2 remapped %d to %d \n",uChar,pCharSpec[i].ucs4));
+				pGlyphSpec[i].glyph = static_cast<FT_UInt>(FT_Get_Char_Index(face,uChar));
+				xxx_UT_DEBUGMSG(("DrawGlyph 2 remapped %d to %d \n",uChar,pGlyphSpec[i].glyph));
 
 			}
 			else
@@ -820,22 +953,47 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 			// it turns out that we don't fit in a short.
 			// or we can keep it in an int array, then transfer to shorts.
 			// that's probably better.
-			pCharSpec[i].x = tdu(xPos);
-			pCharSpec[i].y = currentYoff;
+			idx = _tduX(xPos);
+			if(!isDingbat())
+			{
+				pCharSpec[i].x = idx;
+				pCharSpec[i].y = currentYoff;
+				xxx_UT_DEBUGMSG(("xPos %d idx %d i %d \n",xPos,idx,i));
+			}
+			else
+			{
+				pGlyphSpec[i].x = idx;
+				pGlyphSpec[i].y = currentYoff;
+			}
 			if (i < iLength - 1) {
-				xPos += pCharWidths[i];
+				xPos += pCharWidths[iCharOffset+i];
 			}
 		}
-		
-		XftDrawCharSpec (m_pXftDraw, &m_XftColor, m_pXftFontD, pCharSpec, iLength);
-
+		if(isDingbat())
+		{
+			XftDrawGlyphSpec (m_pXftDraw, &m_XftColor, m_pXftFontD, pGlyphSpec, iLength);
+		}
+		else
+		{
+			XftDrawCharSpec (m_pXftDraw, &m_XftColor, m_pXftFontD, pCharSpec, iLength);
+		}
 		if (pCharSpec != aCharSpec)
 			delete[] pCharSpec;
+		if(isDingbat())
+		{
+			XftUnlockFace (m_pXftFontD);
+			delete [] pGlyphSpec;
+		}
 	}
 }
 
 void GR_UnixGraphics::setFont(GR_Font * pFont)
 {
+	UT_ASSERT(pFont);
+	if(pFont == NULL)
+	{
+		return;
+	}
 	XAP_UnixFontHandle * pUFont = static_cast<XAP_UnixFontHandle *> (pFont);
 
 	// Sometimes we ask gr_UnixGraphics to build big (*BIG*) fonts only to
@@ -866,6 +1024,16 @@ void GR_UnixGraphics::setFont(GR_Font * pFont)
 	//   so I will not meddle with this, but it needs to be
 	//   investigated by someone who knows better -- Tomas
 	
+	// It's worse. Fonts can get deallocated in the cache but have
+    // pointers to them in fp_Run.h. I can detect them here and prevent
+	// crashes but I don't know
+	// how to communicate this back to fp_Run and fully solve this right now 
+	// - Martin
+
+	//	XAP_UnixFont * pUnixFont = pUFont->getUnixFont();
+	//  bool bDealloc = m_pFontManager->isDeallocated(pUnixFont);
+	//  UT_ASSERT(!bDealloc);
+
 	if(m_pFont && (pUFont->getUnixFont() == m_pFont->getUnixFont()) &&
 	   (pUFont->getSize() == m_pFont->getSize()))
 	{
@@ -880,21 +1048,25 @@ void GR_UnixGraphics::setFont(GR_Font * pFont)
 
 	m_pFont = pUFont;
 	char * szUnixFontName = UT_strdup(m_pFont->getUnixFont()->getName());
-	const char * szFontName = UT_lowerString(szUnixFontName);
+	const char * szLCFontName = UT_lowerString(szUnixFontName);
 
-	if (szFontName)
+	if (szLCFontName)
 	{
-		if(strstr(szFontName,"symbol") != NULL)
+		if(strstr(szLCFontName,"symbol") != NULL)
 		{
-			if(strstr(szFontName,"star") != NULL)
+			if(strstr(szLCFontName,"star") != NULL)
 				m_bIsSymbol = false;
 			else
 				m_bIsSymbol = true;
 		}
-		if(strstr(szFontName,"dingbat") != NULL)
+		if(strstr(szLCFontName,"dingbat") != NULL)
 			m_bIsDingbat = true;
+		xxx_UT_DEBUGMSG(("Unix Font name is %s dingbat %d \n",szLCFontName,isDingbat()));
 	}
-	FREEP(szFontName);
+	FREEP(szLCFontName);
+	//	m_bIsSymbol = false;
+	//  m_bIsDingbat = false;
+	
 	m_pXftFontL = m_pFont->getLayoutXftFont();
 	m_pXftFontD = m_pFont->getDeviceXftFont(getZoomPercentage());
 }
@@ -904,7 +1076,7 @@ UT_uint32 GR_UnixGraphics::getFontHeight(GR_Font * fnt)
 	return getFontAscent(fnt)+getFontDescent(fnt);
 }
 
-void GR_UnixGraphics::getCoverage(UT_Vector& coverage)
+void GR_UnixGraphics::getCoverage(UT_NumberVector& coverage)
 {
 	m_pFont->getUnixFont()->getCoverage(coverage);
 }
@@ -917,7 +1089,7 @@ UT_uint32 GR_UnixGraphics::getFontHeight()
 	return getFontAscent()+getFontDescent();
 }
 
-UT_uint32 GR_UnixGraphics::measureUnRemappedChar(const UT_UCSChar c)
+UT_sint32 GR_UnixGraphics::measureUnRemappedChar(const UT_UCSChar c)
 {
 	// measureString() could be defined in terms of measureUnRemappedChar()
 	// but its not (for presumed performance reasons).  Also, a difference
@@ -927,23 +1099,25 @@ UT_uint32 GR_UnixGraphics::measureUnRemappedChar(const UT_UCSChar c)
 	UT_UCSChar newChar;
 	double fWidth;
 
-	if(!m_bIsSymbol && !m_bIsDingbat)
+	if(!isSymbol() && !isDingbat())
 	{
 		newChar = c;
 	}
-	else if(m_bIsSymbol)
+	else if(isSymbol())
 	{
 		newChar = static_cast<UT_UCSChar>(adobeToUnicode(c));
+		xxx_UT_DEBUGMSG(("Measure width of remappedd Symbol %x \n",newChar));
 	}
 	else
 	{
-		newChar = static_cast<UT_UCSChar>(adobeDingbatsToUnicode(c));
+		newChar = c;
 	}
 	// FIXME we should really be getting stuff fromt he font in layout units,
 	// FIXME but we're not smart enough to do that yet
 
 	fWidth = m_pFont->measureUnRemappedChar(newChar, m_pFont->getSize())
 		* ((double)getResolution() / (double)s_getDeviceResolution());
+	xxx_UT_DEBUGMSG(("char %d width = %d \n",newChar,rint(fWidth)));
 	return static_cast<UT_uint32>(rint(fWidth));
 }
 
@@ -1003,7 +1177,6 @@ void GR_UnixGraphics::_setColor(GdkColor & c)
 
 GR_Font * GR_UnixGraphics::getGUIFont(void)
 {
-#ifndef WITH_PANGO
 	if (!m_pFontManager)
 		return NULL;
 
@@ -1024,32 +1197,31 @@ GR_Font * GR_UnixGraphics::getGUIFont(void)
 		m_pFontGUI = new XAP_UnixFontHandle(font, static_cast<UT_uint32>(12*100.0/getZoomPercentage()));
 		UT_ASSERT(m_pFontGUI);
 	}
-#endif
-	// TODO provide PANGO implementation
+
 	return m_pFontGUI;
 }
 
-#ifndef WITH_PANGO
 /**
  * Finds a font which match the family, style, variant, weight and size
  * asked.  It will do a fuzzy match to find the font (using the aliases
  * found in fonts.conf
  */
-GR_Font * GR_UnixGraphics::findFont(const char* pszFontFamily,
-									const char* pszFontStyle,
-									const char* pszFontVariant,
-									const char* pszFontWeight,
-									const char* pszFontStretch,
-									const char* pszFontSize)
+GR_Font * GR_UnixGraphics::_findFont(const char* pszFontFamily,
+									 const char* pszFontStyle,
+									 const char* pszFontVariant,
+									 const char* pszFontWeight,
+									 const char* pszFontStretch,
+									 const char* pszFontSize)
 {
+	xxx_UT_DEBUGMSG(("Find Font _findFont for %s \n",pszFontFamily));
 	XAP_UnixFont* pUnixFont = m_pFontManager->findNearestFont(pszFontFamily, pszFontStyle, pszFontVariant, pszFontWeight,
-															  pszFontStretch, pszFontSize);
+															  pszFontStretch, pszFontSize,this);
 
 	// bury the pointer to our Unix font in a XAP_UnixFontHandle with the correct size.
 	UT_uint32 iSize = static_cast<UT_uint32>(UT_convertToPoints(pszFontSize));
 	XAP_UnixFontHandle* pFont = new XAP_UnixFontHandle(pUnixFont, iSize);
 	UT_ASSERT(pFont);
-
+	xxx_UT_DEBUGMSG(("Return Font name %s \n",pUnixFont->getName())); 
 	return pFont;
 }
 
@@ -1074,7 +1246,7 @@ UT_uint32 GR_UnixGraphics::getFontAscent(GR_Font * fnt)
 	// FIXME we should really be getting stuff fromt he font in layout units,
 	// FIXME but we're not smart enough to do that yet
     // we call s_getDeviceResolution() to avoid zoom
-	return static_cast<UT_uint32>(hndl->getUnixFont()->getAscender(hndl->getSize()) * getResolution() / s_getDeviceResolution() + 0.5);
+	return static_cast<UT_uint32>(hndl->getUnixFont()->getAscender(hndl->getSize()) * getResolution() / s_getDeviceResolution() ); // +0.5);
 }
 
 UT_uint32 GR_UnixGraphics::getFontAscent()
@@ -1092,39 +1264,28 @@ UT_uint32 GR_UnixGraphics::getFontDescent(GR_Font * fnt)
 	XAP_UnixFont* pFont = hndl->getUnixFont();
 	// FIXME we should really be getting stuff fromt he font in layout units,
 	// FIXME but we're not smart enough to do that yet
-	return static_cast<UT_uint32>(pFont->getDescender(hndl->getSize()) * getResolution() / getDeviceResolution() + 0.5);
+	return static_cast<UT_uint32>(pFont->getDescender(hndl->getSize()) * getResolution() / getDeviceResolution() );  // +0.5);
 }
 
 UT_uint32 GR_UnixGraphics::getFontDescent()
 {
 	return getFontDescent(m_pFont);
 }
-#endif //#ifndef WITH_PANGO
 
 
 void GR_UnixGraphics::drawLine(UT_sint32 x1, UT_sint32 y1,
 							   UT_sint32 x2, UT_sint32 y2)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
-	
 	GdkGCValues gcV;
 	gdk_gc_get_values(m_pGC, &gcV);
 	
-	// compensate for the last pixel on straight lines that isn't sometimes drawn by gdk (or better: X)
-	// FIXME: did I exclude all the situations where GDK extends the endpoint?
-	if (!(gcV.join_style == GDK_JOIN_MITER && gcV.cap_style == GDK_CAP_BUTT))
-	{
-		if (y1 == y2) 
-		{
-			x2 += (x1 < x2 ? tlu(1) : -tlu(1));
-		}
-		if (x1 == x2)
-		{
-			y2 += (y1 < y2 ? tlu(1) : -tlu(1));
-		}
-	}
+	UT_sint32 idx1 = _tduX(x1);
+	UT_sint32 idx2 = _tduX(x2);
+
+	UT_sint32 idy1 = _tduY(y1);
+	UT_sint32 idy2 = _tduY(y2);
 	
-	gdk_draw_line(m_pWin, m_pGC, tdu(x1), tdu(y1), tdu(x2), tdu(y2));
+	gdk_draw_line(m_pWin, m_pGC, idx1, idy1, idx2, idy2);
 }
 
 void GR_UnixGraphics::setLineWidth(UT_sint32 iLineWidth)
@@ -1147,14 +1308,17 @@ void GR_UnixGraphics::setLineWidth(UT_sint32 iLineWidth)
 void GR_UnixGraphics::xorLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2,
 							  UT_sint32 y2)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
-	gdk_draw_line(m_pWin, m_pXORGC, tdu(x1), tdu(y1), tdu(x2), tdu(y2));
+	UT_sint32 idx1 = _tduX(x1);
+	UT_sint32 idx2 = _tduX(x2);
+
+	UT_sint32 idy1 = _tduY(y1);
+	UT_sint32 idy2 = _tduY(y2);
+
+	gdk_draw_line(m_pWin, m_pXORGC, idx1, idy1, idx2, idy2);
 }
 
 void GR_UnixGraphics::polyLine(UT_Point * pts, UT_uint32 nPoints)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
-
 	// see bug #303 for what this is about
 
 	GdkPoint * points = static_cast<GdkPoint *>(calloc(nPoints, sizeof(GdkPoint)));
@@ -1162,14 +1326,17 @@ void GR_UnixGraphics::polyLine(UT_Point * pts, UT_uint32 nPoints)
 
 	for (UT_uint32 i = 0; i < nPoints; i++)
 	{
-		points[i].x = tdu(pts[i].x);
+		UT_sint32 idx = _tduX(pts[i].x);
+		points[i].x = idx;
 		// It seems that Windows draws each pixel along the the Y axis
 		// one pixel beyond where GDK draws it (even though both coordinate
 		// systems start at 0,0 (?)).  Subtracting one clears this up so
 		// that the poly line is in the correct place relative to where
 		// the rest of GR_UnixGraphics:: does things (drawing text, clearing
 		// areas, etc.).
-		points[i].y = tdu(pts[i].y) - 1;
+		UT_sint32 idy1 = _tduY(pts[i].y);
+
+		points[i].y = idy1 - 1;
 	}
 
 	gdk_draw_lines(m_pWin, m_pGC, points, nPoints);
@@ -1179,10 +1346,15 @@ void GR_UnixGraphics::polyLine(UT_Point * pts, UT_uint32 nPoints)
 
 void GR_UnixGraphics::invertRect(const UT_Rect* pRect)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
 	UT_ASSERT(pRect);
-	gdk_draw_rectangle(m_pWin, m_pXORGC, 1, tdu(pRect->left), tdu(pRect->top),
-			   tdu(pRect->width), tdu(pRect->height));
+
+	UT_sint32 idy = _tduY(pRect->top);
+	UT_sint32 idx = _tduX(pRect->left);
+	UT_sint32 idw = _tduR(pRect->width);
+	UT_sint32 idh = _tduR(pRect->height);
+
+	gdk_draw_rectangle(m_pWin, m_pXORGC, 1, idx, idy,
+			   idw, idh);
 }
 
 void GR_UnixGraphics::setClipRect(const UT_Rect* pRect)
@@ -1191,11 +1363,14 @@ void GR_UnixGraphics::setClipRect(const UT_Rect* pRect)
 	if (pRect)
 	{
 		GdkRectangle r;
-
-		r.x = tdu(pRect->left);
-		r.y = tdu(pRect->top);
-		r.width = tdu(pRect->width);
-		r.height = tdu(pRect->height);
+		UT_sint32 idy = _tduY(pRect->top);
+		UT_sint32 idx = _tduX(pRect->left);
+		//UT_sint32 idw = _tduR(pRect->width);
+		//UT_sint32 idh = _tduR(pRect->height);
+		r.x = idx;
+		r.y = idy;
+		r.width = _tduR(pRect->width);
+		r.height = _tduR(pRect->height);
 
 		gdk_gc_set_clip_rectangle(m_pGC, &r);
 		gdk_gc_set_clip_rectangle(m_pXORGC, &r);
@@ -1235,7 +1410,6 @@ void GR_UnixGraphics::setClipRect(const UT_Rect* pRect)
 void GR_UnixGraphics::fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 							   UT_sint32 w, UT_sint32 h)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
 	// save away the current color, and restore it after we fill the rect
 	GdkGCValues gcValues;
 	GdkColor oColor;
@@ -1256,8 +1430,11 @@ void GR_UnixGraphics::fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 	gdk_colormap_alloc_color(m_pColormap, &nColor, FALSE, TRUE);
 
 	gdk_gc_set_foreground(m_pGC, &nColor);
-
- 	gdk_draw_rectangle(m_pWin, m_pGC, 1, tdu(x), tdu(y), tdu(w), tdu(h));
+	UT_sint32 idx = _tduX(x);
+	UT_sint32 idy = _tduY(y);
+	UT_sint32 idw = _tduR(w);
+	UT_sint32 idh = _tduR(h);
+ 	gdk_draw_rectangle(m_pWin, m_pGC, 1, idx, idy, idw, idh);
 
 	gdk_gc_set_foreground(m_pGC, &oColor);
 }
@@ -1280,7 +1457,6 @@ void GR_UnixGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 	UT_sint32 iddy = labs(ddy);
 	bool bEnableSmooth = m_pApp->isSmoothScrollingEnabled();
 	bEnableSmooth = bEnableSmooth && (iddy < 30) && (ddx == 0);
-#if 1
 	if(bEnableSmooth)
 	{
 		if(ddy < 0)
@@ -1304,118 +1480,7 @@ void GR_UnixGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 	{
 		gdk_window_scroll(m_pWin,ddx,ddy);
 	}
-#else
-	GdkRectangle  xRect;
-	GdkRectangle  yRect;
-	xRect.height = 0;
-	yRect.height = 0;
-	UT_sint32 winWidth,winHeight,x,y,depth;
-	gdk_window_get_geometry(m_pWin,&x,&y,&winWidth,&winHeight,&depth);
-	//
-	// Assume (0,0) is top left corner
-	//
-	if(ddy < 0)
-	{
-		//
-		// We're moving up so height is increased top is reduced.
-		//
-		yRect.height = -ddy;
-		yRect.y = winHeight + ddy;
-		yRect.x = 0;
-		yRect.width = winWidth; 
-	}
-	if(ddy > 0)
-	{
-		//
-		// moving down
-		//
-		yRect.height = ddy;
-		yRect.y = 0;
-		yRect.x = 0;
-		yRect.width = winWidth; 
-	}
-	if(ddx < 0)
-	{
-		//
-		// Moving left
-		//
-		xRect.height = winHeight;
-		xRect.y = 0;
-		xRect.x = winWidth;
-		xRect.width = -ddx; 
-	}
-	if(ddx > 0)
-	{
-		//
-		// We're moving right
-		//
-		xRect.height = winHeight;
-		xRect.y = 0;
-		xRect.x = 0;
-		yRect.width = -ddx; 
-	}
-	if (ddy > 0)
-    {
-		if (ddy < winHeight)
-		{
-			if(bEnableSmooth)
-			{
-				UT_sint32 i = 0;
-				for(i = 0; i< iddy; i++)
-				{
-					gdk_draw_drawable(m_pWin, m_pGC, m_pWin,0, 0,
-								  0, ddy, winWidth, winHeight - i);
-				}
-			}
-			else
-			{
-				gdk_draw_drawable(m_pWin, m_pGC, m_pWin,0, 0,
-								 0, ddy, winWidth, winHeight - ddy);
-			}
-		}
-		gdk_window_invalidate_rect(m_pWin,&yRect,TRUE);
-    }
-	else if (ddy < 0)
-    {
-		if (ddy >= -winHeight)
-		{
-			if(bEnableSmooth)
-			{
-				UT_sint32 i = 0;
-				for(i = 0; i< iddy; i++)
-				{
-					gdk_draw_drawable(m_pWin, m_pGC, m_pWin,0, i,
-								  0, ddy, winWidth, winHeight - i);
-				}
-			}
-			else
-			{
-				gdk_draw_drawable(m_pWin, m_pGC, m_pWin,0, -ddy,
-								  0, 0, winWidth, winHeight + ddy);
-			}
-		}
-		gdk_window_invalidate_rect(m_pWin,&yRect,TRUE);
-    }
-
-	if (ddx > 0)
-    {
-		if (ddx < winWidth)
-		{
-			gdk_draw_drawable(m_pWin, m_pGC, m_pWin, 0, 0,
-								  ddx, 0, winWidth - ddx, winHeight);
-		}
-	  		gdk_window_invalidate_rect(m_pWin,&xRect,TRUE);
-    }
-	else if (ddx < 0)
-    {
-		if (ddx >= -winWidth)
-		{
-			gdk_draw_drawable(m_pWin, m_pGC, m_pWin, -ddx, 0,
-								  0, 0, winWidth + ddx, winHeight);
-		}
-		gdk_window_invalidate_rect(m_pWin,&xRect,TRUE);
-    }
-#endif
+	setExposePending(true);
 }
 
 void GR_UnixGraphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
@@ -1423,8 +1488,6 @@ void GR_UnixGraphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 						  UT_sint32 width, UT_sint32 height)
 {
 	GR_CaretDisabler caretDisabler(getCaret());
-	//	gdk_draw_drawable(m_pWin, m_pGC, m_pWin, tdu(x_dest), tdu(y_dest),
-	//				  tdu(x_src), tdu(y_src), tdu(width), tdu(height));
    	gdk_draw_drawable(m_pWin, m_pGC, m_pWin, tdu(x_src), tdu(y_src),
    				  tdu(x_dest), tdu(y_dest), tdu(width), tdu(height));
 }
@@ -1432,8 +1495,6 @@ void GR_UnixGraphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 void GR_UnixGraphics::clearArea(UT_sint32 x, UT_sint32 y,
 				UT_sint32 width, UT_sint32 height)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
-
 	if (width > 0)
 	{
 		static const UT_RGBColor clrWhite(255,255,255);
@@ -1461,7 +1522,12 @@ bool GR_UnixGraphics::endPrint(void)
 }
 
 // gdk-pixbuf based routines
-
+/*!
+ * Create a new image from the Raster rgba byte buffer defined by pBB.
+ * The dimensions of iWidth and iHeight are in logical units but the image
+ * doesn't scale if the resolution or zoom changes. Instead you must create
+ * a new image.
+ */
 GR_Image* GR_UnixGraphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB, UT_sint32 iWidth, UT_sint32 iHeight, GR_Image::GRType iType)
 {
    	GR_Image* pImg = NULL;
@@ -1476,10 +1542,12 @@ GR_Image* GR_UnixGraphics::createNewImage(const char* pszName, const UT_ByteBuf*
 // and that values < than the alpha threshold are painted as 0s
 // this seems to work for me, so I'm happy - Dom
 #define ABI_ALPHA_THRESHOLD 100
-
+/*!
+ * Draw the specified image at the location specified in local units 
+ * (xDest,yDest). xDest and yDest are in logical units.
+ */
 void GR_UnixGraphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
 	UT_ASSERT(pImg);
 
    	GR_UnixImage * pUnixImage = static_cast<GR_UnixImage *>(pImg);
@@ -1489,8 +1557,10 @@ void GR_UnixGraphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest
 
    	UT_sint32 iImageWidth = pUnixImage->getDisplayWidth();
    	UT_sint32 iImageHeight = pUnixImage->getDisplayHeight();
+	UT_sint32 idx = _tduX(xDest);
+	UT_sint32 idy = _tduY(yDest);
 
-	xDest = tdu(xDest); yDest = tdu(yDest);
+	xDest = idx; yDest = idy;
 
 	if (gdk_pixbuf_get_has_alpha (image))
 		gdk_draw_pixbuf (m_pWin, NULL, image,
@@ -1624,8 +1694,24 @@ void GR_UnixGraphics::setCursor(GR_Graphics::Cursor c)
 	case GR_CURSOR_VLINE_DRAG:
 		cursor_number = GDK_SB_H_DOUBLE_ARROW;
 		break;
-	}
 
+	case GR_CURSOR_CROSSHAIR:
+		cursor_number = GDK_CROSSHAIR;
+		break;
+
+	case GR_CURSOR_DOWNARROW:
+		cursor_number = GDK_SB_DOWN_ARROW;
+		break;
+
+	case GR_CURSOR_DRAGTEXT:
+		cursor_number = GDK_TARGET;
+		break;
+
+	case GR_CURSOR_COPYTEXT:
+		cursor_number = GDK_DRAPED_BOX;
+		break;
+	}
+	xxx_UT_DEBUGMSG(("cursor set to %d  gdk %d \n",c,cursor_number));
 	GdkCursor * cursor = gdk_cursor_new(cursor_number);
 	gdk_window_set_cursor(m_pWin, cursor);
 	gdk_cursor_unref(cursor);
@@ -1651,18 +1737,30 @@ void GR_UnixGraphics::setColor3D(GR_Color3D c)
 	_setColor(m_3dColors[c]);
 }
 
+bool GR_UnixGraphics::getColor3D(GR_Color3D name, UT_RGBColor &color)
+{
+	if (m_bHave3DColors) {
+		color.m_red = m_3dColors[name].red >> 8;
+		color.m_grn = m_3dColors[name].green >> 8;
+		color.m_blu =m_3dColors[name].blue >> 8;
+		return true;
+	}
+	return false;
+}
+
 void GR_UnixGraphics::init3dColors(GtkStyle * pStyle)
 {
-	m_3dColors[CLR3D_Foreground] = pStyle->fg[GTK_STATE_NORMAL];
+	m_3dColors[CLR3D_Foreground] = pStyle->text[GTK_STATE_NORMAL];
 	m_3dColors[CLR3D_Background] = pStyle->bg[GTK_STATE_NORMAL];
 	m_3dColors[CLR3D_BevelUp]    = pStyle->light[GTK_STATE_NORMAL];
 	m_3dColors[CLR3D_BevelDown]  = pStyle->dark[GTK_STATE_NORMAL];
 	m_3dColors[CLR3D_Highlight]  = pStyle->bg[GTK_STATE_PRELIGHT];
+
+	m_bHave3DColors = true;
 }
 
 void GR_UnixGraphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
 	UT_ASSERT(c < COUNT_3D_COLORS);
 	gdk_gc_set_foreground(m_pGC, &m_3dColors[c]);
 	gdk_draw_rectangle(m_pWin, m_pGC, 1, tdu(x), tdu(y), tdu(w), tdu(h));
@@ -1676,7 +1774,6 @@ void GR_UnixGraphics::fillRect(GR_Color3D c, UT_Rect &r)
 
 void GR_UnixGraphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
 {
-	GR_CaretDisabler caretDisabler(getCaret());
 	// save away the current color, and restore it after we draw the polygon
 	GdkGCValues gcValues;
 	GdkColor oColor;
@@ -1702,8 +1799,10 @@ void GR_UnixGraphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
     UT_ASSERT(points);
 
     for (UT_uint32 i = 0;i < nPoints;i++){
-        points[i].x = tdu(pts[i].x);
-        points[i].y = tdu(pts[i].y);
+		UT_sint32 idx = _tduX(pts[i].x);
+        points[i].x = idx;
+		UT_sint32 idy = _tduY(pts[i].y);
+        points[i].y = idy;
     }
 	gdk_draw_polygon(m_pWin, m_pGC, 1, points, nPoints);
 	delete[] points;
@@ -1711,7 +1810,6 @@ void GR_UnixGraphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
 	gdk_gc_set_foreground(m_pGC, &oColor);
 }
 
-#ifndef WITH_PANGO
 //////////////////////////////////////////////////////////////////
 // This is a static method in the GR_Font base class implemented
 // in platform code.
@@ -1735,22 +1833,52 @@ void GR_Font::s_getGenericFontProperties(const char * /*szFontName*/,
 	*pfp = FP_Unknown;
 	*pbTrueType = true;
 }
-#endif
+
+
+/*!
+ * Take a screenshot of the graphics and convert it to an image.
+ */
+GR_Image * GR_UnixGraphics::genImageFromRectangle(const UT_Rect &rec)
+{
+	UT_sint32 idx = _tduX(rec.left);
+	UT_sint32 idy = _tduY(rec.top);
+	UT_sint32 idw = _tduR(rec.width);
+	UT_sint32 idh = _tduR(rec.height);
+	GdkColormap* cmp = gdk_colormap_get_system();
+	GdkPixbuf * pix = gdk_pixbuf_get_from_drawable(NULL,
+												   m_pWin,
+												   cmp,
+												   idx, idy, 0, 0,
+												   idw, idh);
+	
+	UT_return_val_if_fail(pix, NULL);
+
+	GR_UnixImage * pImg = new GR_UnixImage("ScreenShot");
+	pImg->m_image = pix;
+	return static_cast<GR_Image *>(pImg);
+}
 
 void GR_UnixGraphics::saveRectangle(UT_Rect & r, UT_uint32 iIndx)
 {
-	void * oldR = NULL;
-	m_vSaveRect.setNthItem(iIndx, static_cast<void*>(new UT_Rect(r)),&oldR);
-	if(oldR)
-		delete static_cast<UT_Rect*>(oldR);
+	UT_Rect* oldR = NULL;	
+
+	m_vSaveRect.setNthItem(iIndx, new UT_Rect(r),&oldR);
+	if(oldR) {
+		delete oldR;
+	}
+
+	GdkPixbuf * oldC = NULL;
+	UT_sint32 idx = _tduX(r.left);
+	UT_sint32 idy = _tduY(r.top);
+	UT_sint32 idw = _tduR(r.width);
+	UT_sint32 idh = _tduR(r.height);
 
-	void * oldC = NULL;
 	GdkPixbuf * pix = gdk_pixbuf_get_from_drawable(NULL,
 												   m_pWin,
 												   NULL,
-												   tdu(r.left), tdu(r.top), 0, 0,
-												   tdu(r.width), tdu(r.height));
-	m_vSaveRectBuf.setNthItem(iIndx, static_cast<void*>(pix), &oldC);
+												   idx, idy, 0, 0,
+												   idw, idh);
+	m_vSaveRectBuf.setNthItem(iIndx, pix, &oldC);
 
 	if(oldC)
 		g_object_unref (G_OBJECT (oldC));
@@ -1758,11 +1886,41 @@ void GR_UnixGraphics::saveRectangle(UT_Rect & r, UT_uint32 iIndx)
 
 void GR_UnixGraphics::restoreRectangle(UT_uint32 iIndx)
 {
-	UT_Rect * r = static_cast<UT_Rect*>(m_vSaveRect.getNthItem(iIndx));
-	GdkPixbuf *p = static_cast<GdkPixbuf *>(m_vSaveRectBuf.getNthItem(iIndx));
+	UT_Rect * r = m_vSaveRect.getNthItem(iIndx);
+	GdkPixbuf *p = m_vSaveRectBuf.getNthItem(iIndx);
+	UT_sint32 idx = _tduX(r->left);
+	UT_sint32 idy = _tduY(r->top);
+
 
 	if (p && r)
 		gdk_draw_pixbuf (m_pWin, NULL, p, 0, 0,
-						 tdu(r->left), tdu(r->top),
+						 idx, idy,
 						 -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
 }
+
+void GR_UnixGraphics::_beginPaint ()
+{
+}
+
+void GR_UnixGraphics::_endPaint ()
+{
+}
+
+
+GR_Graphics *   GR_UnixGraphics::graphicsAllocator(GR_AllocInfo& allocInfo)
+{
+
+	if (allocInfo.getType() == GRID_UNIX_NULL)
+		{
+			return UnixNull_Graphics::graphicsAllocator(allocInfo);
+		}
+	GR_UnixAllocInfo &allocator = (GR_UnixAllocInfo&)allocInfo;
+	if (allocator.m_win) 
+		{
+			return new GR_UnixGraphics(allocator.m_win, allocator.m_fontManager, XAP_App::getApp());
+		}
+	else 
+		{
+		return new GR_UnixGraphics(allocator.m_pixmap, allocator.m_fontManager, XAP_App::getApp(), allocator.m_usePixmap);
+	}
+}
