@@ -62,16 +62,6 @@
 
 /*****************************************************************/
 
-const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
-										 const char* pszFontStyle,
-										 const char* pszFontVariant,
-										 const char* pszFontWeight,
-										 const char* pszFontStretch,
-										 const char* pszFontSize)
-{
-	return pszFontFamily;
-}
-
 // A small helper class
 class private_FontReverter
 {
@@ -897,14 +887,14 @@ void GR_Win32Graphics::fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 	RECT r;
 	r.left = (UT_sint32)((double)_tduX(x) * m_fXYRatio);
 	r.top = _tduY(y);
-	r.right = _tduX(x + w);
-	r.bottom =(UT_sint32)((double) _tduY(y + h) * m_fXYRatio);
-	w=(UT_sint32)((double)_tduR(w) * m_fXYRatio);
-	h=_tduR(h);
+	r.right = (UT_sint32)(_tduX(x+w) * m_fXYRatio);
+	r.bottom = _tduY(y+h);
 
 	COLORREF clr = RGB(c.m_red, c.m_grn, c.m_blu);
 
 	#ifdef GR_GRAPHICS_DEBUG
+	w=(UT_sint32)((double)_tduR(w) * m_fXYRatio);
+	h=_tduR(h);
 	UT_DEBUGMSG(("GR_Win32Graphics::fillRect %x %u %u %u %u\n",  clr, r.left, r.top, w, h));	
 	#endif
 		
@@ -943,6 +933,10 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 		pDevMode->dmFields = DM_ORIENTATION;
 		pDevMode->dmOrientation = (bPortrait) ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
 		GlobalUnlock(m_hDevMode);
+
+		// call DocumentProperties() on the DEVMODE to ensure changes propagate down into
+		// the private part
+		fixDevMode(m_hDevMode);
 		
 		pDevMode = (DEVMODE*) GlobalLock(m_hDevMode);
 		ResetDC(m_hdc, pDevMode);
@@ -1045,7 +1039,21 @@ void GR_Win32Graphics::clearArea(UT_sint32 x, UT_sint32 y, UT_sint32 width, UT_s
 	r.right = r.left + width;
 	r.bottom = r.top + height;
 
+#if 1
+	// for the sake of consistency we should be using the same method as the fillRect()
+	// does; however, we already have brush created, so FillRect is probably quicker here
+	// (someone should do some profiling on this one day)
 	FillRect(m_hdc, &r, m_hClearBrush);
+#else
+	// this is the method used by fillRect(); if we were to use it, it we should
+	// cache lb.lbColor from inside setBrush() to avoid calling GetObject() all the time
+	LOGBRUSH lb;
+	GetObject(m_hClearBrush, sizeof(LOGBRUSH), &lb);
+	
+	const COLORREF cr = ::SetBkColor(m_hdc,  lb.lbColor);
+	::ExtTextOut(m_hdc, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
+	::SetBkColor(m_hdc, cr);
+#endif
 }
 
 void GR_Win32Graphics::invertRect(const UT_Rect* pRect)
@@ -1314,7 +1322,6 @@ void GR_Win32Graphics::init3dColors(void)
 void GR_Win32Graphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h)
 {
 	UT_ASSERT(c < COUNT_3D_COLORS);
-	HBRUSH hBrush = CreateSolidBrush(m_3dColors[c]); 
 
 	#ifdef GR_GRAPHICS_DEBUG
 	UT_DEBUGMSG(("GR_Win32Graphics::fillRect GR_Color3D  %x %u %u %u %u\n",  c, x, y, w, h));	
@@ -1323,11 +1330,17 @@ void GR_Win32Graphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint3
 	RECT r;
 	r.left = (UT_sint32)((double)_tduX(x) * m_fXYRatio);
 	r.top = _tduY(y);
-	r.right = (UT_sint32)((double)_tduX(x + w) * m_fXYRatio);
-	r.bottom = _tduY(y + h);
+	r.right = (UT_sint32)((double)_tduX(x+w) * m_fXYRatio);
+	r.bottom = _tduY(y+h);
 
-	FillRect(m_hdc, &r, hBrush);
-	DeleteObject(hBrush);
+	// This might look wierd (and I think it is), but it's MUCH faster.
+	// CreateSolidBrush is dog slow.
+	HDC hdc = m_hdc;
+	COLORREF clr = RGB(GetRValue(m_3dColors[c]), GetGValue(m_3dColors[c]), GetBValue(m_3dColors[c]));
+	
+	const COLORREF cr = ::SetBkColor(hdc,  clr);
+	::ExtTextOut(hdc, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
+	::SetBkColor(hdc, cr);
 	setExposePending(false);
 }
 
@@ -1661,6 +1674,100 @@ UT_sint32 GR_Win32Font::measureUnremappedCharForCache(UT_UCSChar cChar) const
 #endif
 }
 
+/*!
+    UT_Rect of glyph in Logical units.
+	rec.left = bearing Left (distance from origin to start)
+	rec.width = width of the glyph
+	rec.top = distance from the origin to the top of the glyph
+	rec.height = total height of the glyph
+
+    This function will only work on win NT -- check the return value before doing anything
+    with the rectangle (win9x implementation would have to use GetGlyphOutlineA() and
+    convert the UT_UCS4Char to an appropriate ansi value 
+*/
+bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
+{
+	UT_return_val_if_fail( pG, false );
+	GR_Win32Graphics * pWin32Gr = (GR_Win32Graphics *)pG;
+
+	// we do all measurements on the printer DC and only later scale down to display to
+	// ensure WYSIWYG behaviour
+	HDC hdc = getPrimaryHDC();
+	HDC printDC = pWin32Gr->getPrintDC() ? pWin32Gr->getPrintDC() : hdc;
+	
+	GLYPHMETRICS gm;
+	MAT2 m = {{0,1}, {0,0}, {0,0}, {0,1}};
+
+	int nLogPixelsY      = GetDeviceCaps(hdc, LOGPIXELSY);    // resolution of primary device
+	int nPrintLogPixelsY = GetDeviceCaps(printDC, LOGPIXELSY);// resolution of printer
+
+	int nLogPixelsX      = GetDeviceCaps(hdc, LOGPIXELSX);
+	int nPrintLogPixelsX = GetDeviceCaps(printDC, LOGPIXELSX);
+
+	UT_uint32 pixels = getUnscaledHeight();
+
+	// scale the pixel size to the printer resolution and get the font
+	if(!m_bGUIFont)
+		pixels = MulDiv(pixels, nPrintLogPixelsY, 72);
+	
+	HFONT pFont = getFontFromCache(pixels, false, 100);
+	if (!pFont)
+	{
+		fetchFont(pixels);
+		pFont = getFontFromCache(pixels, false, 100);
+	}
+
+	// select our font into the printer DC
+	HGDIOBJ hRet = SelectObject(printDC, pFont);
+	UT_return_val_if_fail( hRet != (void*)GDI_ERROR, false);
+		
+	if (printDC != m_hdc)
+	{
+		// invalidate cached info when we change hdc's.
+		// this is probably unnecessary except when
+		// sharing a font with screen and printer.
+		_clearAnyCachedInfo();
+		m_hdc = printDC;
+	}
+	
+	DWORD iRet = GDI_ERROR;
+	
+	if (UT_IsWinNT())
+	{
+		iRet = GetGlyphOutlineW(printDC, (UINT)g, GGO_METRICS, &gm, 0, NULL, &m);
+	}
+	else
+	{
+		// TODO: an UT_UCS4Char -> ANSI conversion here, then call GetGlyphOutlineA() ...
+		UT_return_val_if_fail( UT_NOT_IMPLEMENTED, false );
+	}
+	
+	if(GDI_ERROR == iRet)
+		return false;
+
+	rec.left   = gm.gmptGlyphOrigin.x;
+	rec.top    = gm.gmptGlyphOrigin.y;
+	rec.width  = gm.gmBlackBoxX;
+	rec.height = gm.gmBlackBoxY;
+	
+	// if the primary DC is not the printer, we now have to scale the metrics to the
+	// screen resolution
+	if(nLogPixelsY != nPrintLogPixelsY)
+	{
+		rec.height  = MulDiv(rec.height, nLogPixelsY, nPrintLogPixelsY);
+		rec.top  = MulDiv(rec.top, nLogPixelsY, nPrintLogPixelsY);
+	}
+
+	if(nLogPixelsX != nPrintLogPixelsX)
+	{
+		rec.width  = MulDiv(rec.width, nLogPixelsX, nPrintLogPixelsX);
+		rec.left  = MulDiv(rec.left, nLogPixelsX, nPrintLogPixelsX);
+	}
+
+	UT_DEBUGMSG(("gr_Win32Graphics::glyphBox(l=%d, t=%d, w=%d, h=%d\n", rec.left, rec.top, rec.width, rec.height));
+	return true;
+}
+
 /*!
 	Implements a GR_CharWidths.
 	Override if you which to instanciate a subclass.
@@ -2130,6 +2237,7 @@ GR_Graphics * GR_Win32Graphics::getPrinterGraphics(const char * pPrinterName,
 	// we will use '-' as a shortcut for default printer (the --print command has to have
 	// a parameter)
 	char * pPN = UT_strcmp("-", pPrinterName) == 0 ? NULL : (char *) pPrinterName;
+	const char * pDriver = UT_IsWinNT() ? "WINSPOOL" : NULL;
 	
 	if(!pPN)
 	{
@@ -2139,7 +2247,7 @@ GR_Graphics * GR_Win32Graphics::getPrinterGraphics(const char * pPrinterName,
 
 	_test_and_cleanup(pPN);
 
-	hPDC = CreateDC(NULL, pPN, NULL, NULL);
+	hPDC = CreateDC(pDriver, pPN, NULL, NULL);
 
 	_test_and_cleanup(hPDC);
 	
@@ -2252,7 +2360,7 @@ bool GR_Win32Graphics::fixDevMode(HGLOBAL hDevMode)
 	}
 	
 	// now get the printer driver to merge the data in the public section into its private part
-	dwRet = DocumentProperties(NULL,hPrinter, (char*)& pDM->dmDeviceName, pDM, pDM, DM_IN_BUFFER | DM_OUT_BUFFER);
+	dwRet = DocumentProperties(NULL,hPrinter, (char*)& pDM->dmDeviceName, pDM, pDM, DM_OUT_BUFFER | DM_IN_BUFFER);
 
 	// free what needs be ...
 	ClosePrinter(hPrinter);
