@@ -176,6 +176,19 @@ void XAP_CocoaFont::getCoverage(UT_NumberVector& coverage)
 	coverage = *_m_coverage;
 }
 
+// UT_Rect of glyph in Logical units.
+// rec.left = bearing Left (distance from origin to start)
+// rec.width = width of the glyph
+// rec.top = distance from the origin to the top of the glyph
+// rec.height = total height of the glyph
+
+bool XAP_CocoaFont::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics *)
+{
+  // FIXME: Write the code for this!
+  UT_ASSERT(0);
+  return false;
+}
+
 UT_sint32 XAP_CocoaFont::measureUnremappedCharForCache(UT_UCSChar cChar) const
 {
 	if (m_fontForCache == nil) {
@@ -241,6 +254,7 @@ typedef struct {
 	const char *	description;
 } UT_SpecialCharacter;
 
+#if 0
 static UT_SpecialCharacter s_unicode_extra[] = {
 	{ 0x00A0, "Non-breaking space" },
 	{ 0x2206, "Increment (Delta)"  },
@@ -248,6 +262,7 @@ static UT_SpecialCharacter s_unicode_extra[] = {
 	{ 0x00B5, "Micro sign (mu)"    },
 	{ 0x2044, "Fraction slash"     }
 };
+#endif
 
 /* Unicode mapping for Symbol (provided by Adobe, available from http://www.unicode.org/)
  */
