@@ -50,6 +50,7 @@
 #include "ut_string.h"
 #include "xap_Frame.h"
 #include "spell_manager.h"
+#include "gr_EmbedManager.h"
 
 #define REDRAW_UPDATE_MSECS	500
 
@@ -72,39 +73,59 @@ const char * s_FootnoteTypeDesc[] = {
 };
 
 FL_DocLayout::FL_DocLayout(PD_Document* doc, GR_Graphics* pG)
+  : m_pG(pG),
+    m_pDoc(doc),
+    m_pView(NULL),
+    m_lid((PL_ListenerId)-1),
+    m_pFirstSection(NULL),
+    m_pLastSection(NULL),
+    m_pPendingBlockForSpell(NULL),
+    m_pPendingWordForSpell(NULL),
+    m_bSpellCheckCaps(true),
+    m_bSpellCheckNumbers(true),
+    m_bSpellCheckInternet(true),
+    m_bAutoSpellCheck(true),
+    m_uDocBackgroundCheckReasons(0),
+    m_bStopSpellChecking(false),
+    m_bImSpellCheckingNow(false),
+    m_pPendingBlockForSmartQuote(NULL),
+    m_uOffsetForSmartQuote(0),
+    m_pBackgroundCheckTimer(NULL),
+    m_pPrefs(NULL),
+    m_pRedrawUpdateTimer(NULL),
+    m_iSkipUpdates(0),
+    m_bDeletingLayout(false),
+    m_bisLayoutFilling(false),
+    m_iRedrawCount(0),
+    m_FootnoteType(FOOTNOTE_TYPE_NUMERIC),
+    m_iFootnoteVal(1),
+    m_bRestartFootSection(false),
+    m_bRestartFootPage(false),
+    m_iEndnoteVal(1),
+    m_EndnoteType(FOOTNOTE_TYPE_NUMERIC_SQUARE_BRACKETS),
+    m_bRestartEndSection(false),
+    m_bPlaceAtDocEnd(false),
+    m_bPlaceAtSecEnd(true),
+    m_iGraphicTick(0), 
+    m_iDocSize(0),
+    m_iFilled(0),
+    m_bSpellCheckInProgress(false),
+    m_bAutoGrammarCheck(false),
+    m_PendingBlockForGrammar(NULL),
+    m_iGrammarCount(0),
+    m_bFinishedInitialCheck(false),
+    m_iPrevPos(0)
 {
-	m_pDoc = doc;
-	m_pG = pG;
-	m_pView = NULL;
-
-	m_pBackgroundCheckTimer = NULL;
-	m_pPendingBlockForSpell = NULL;
-	m_pPendingWordForSpell = NULL;
-	m_pPendingBlockForSmartQuote = NULL;
-	m_uOffsetForSmartQuote = 0;
-	m_pFirstSection = NULL;
-	m_pLastSection = NULL;
-	m_bSpellCheckCaps = true;
-	m_bSpellCheckNumbers = true;
-	m_bSpellCheckInternet = true;
-	m_bAutoSpellCheck = true;
-	m_pPrefs = NULL;
-	m_bStopSpellChecking = false;
-	m_bImSpellCheckingNow = false;
-	m_uDocBackgroundCheckReasons = 0;
-	m_iSkipUpdates = 0;
-	m_bDeletingLayout = false;
-	setLayoutIsFilling(false);
-	m_lid = (PL_ListenerId)-1;
-	m_iGraphicTick = 0;
+#ifdef FMT_TEST
+        m_pDocLayout = this;
+#endif
+        setLayoutIsFilling(false),
 	m_pRedrawUpdateTimer = UT_Timer::static_constructor(_redrawUpdate, this, m_pG);
 	if (m_pRedrawUpdateTimer)
 	{
 		m_pRedrawUpdateTimer->set(REDRAW_UPDATE_MSECS);
 		m_pRedrawUpdateTimer->start();
 	}
-	m_iFilled = 0;
-	m_bSpellCheckInProgress	= false;
 	
 	// TODO the following (both the new() and the addListener() cause
 	// TODO malloc's to occur.  we are currently inside a constructor
@@ -115,22 +136,8 @@ FL_DocLayout::FL_DocLayout(PD_Document* doc, GR_Graphics* pG)
 	m_pDoc->disableListUpdates();
 
 	strncpy(m_szCurrentTransparentColor,static_cast<const char *>(XAP_PREF_DEFAULT_ColorForTransparent),9);
-
-#ifdef FMT_TEST
-	m_pDocLayout = this;
-#endif
-	m_iRedrawCount = 0;
 	m_vecFootnotes.clear();
 	m_vecEndnotes.clear();
-	m_FootnoteType = FOOTNOTE_TYPE_NUMERIC;
-	m_iFootnoteVal = 1;
-	m_bRestartFootSection = false;
-	m_bRestartFootPage = false;
-	m_iEndnoteVal = 1;
-	m_EndnoteType = FOOTNOTE_TYPE_NUMERIC_SQUARE_BRACKETS;
-    m_bRestartEndSection = false;
-	m_bPlaceAtDocEnd = false;
-	m_bPlaceAtSecEnd = true;
 
 }
 
@@ -153,18 +160,6 @@ FL_DocLayout::~FL_DocLayout()
 	{
 		m_bStopSpellChecking = true;
 		m_pBackgroundCheckTimer->stop();
-#ifdef __BEOS__
-		m_pBackgroundCheckTimer->start();	// is already stopped
-											// so we don't go out of this while
-#endif
-		while(m_bImSpellCheckingNow == true)
-		{
-#ifdef __BEOS__
-			/* On BeOS, we must release the cpu as timers are
-			   run on separate threads */
-			UT_usleep(10000);
-#endif
-		}
 	}
 
 	DELETEP(m_pBackgroundCheckTimer);
@@ -196,6 +191,42 @@ FL_DocLayout::~FL_DocLayout()
 		delete m_pFirstSection;
 		m_pFirstSection = pNext;
 	}
+	UT_VECTOR_PURGEALL(GR_EmbedManager *,m_vecEmbedManager);
+}
+
+
+/*!
+ * Get an embedManager of the requested Type.
+ */
+GR_EmbedManager * FL_DocLayout::getEmbedManager(const char * szEmbedType)
+{
+  // Look in the current collection first.
+  UT_uint32 i = 0;
+  GR_EmbedManager * pDefault = NULL;
+  GR_EmbedManager * pEmbed = NULL;
+  for(i=0; i< m_vecEmbedManager.getItemCount(); i++)
+    {
+      pEmbed = m_vecEmbedManager.getNthItem(i);
+      if(UT_strcmp(pEmbed->getObjectType(),szEmbedType) == 0)
+	{
+	  return pEmbed;
+	}
+      if(UT_strcmp(pEmbed->getObjectType(),"default") == 0)
+	{
+	  pDefault = pEmbed;
+	}
+    }
+  pEmbed = XAP_App::getApp()->getEmbeddableManager(m_pG,szEmbedType);
+  if((UT_strcmp(pEmbed->getObjectType(),"default") == 0) && pDefault != NULL)
+    {
+      delete pEmbed;
+      return pDefault;
+    }
+  UT_DEBUGMSG(("Got mamanger of type %s \n",pEmbed->getObjectType()));
+  m_vecEmbedManager.addItem(pEmbed);
+  pEmbed->initialize();
+  
+  return pEmbed;
 }
 
 /*! 
@@ -458,6 +489,9 @@ void FL_DocLayout::fillLayouts(void)
 	UT_ASSERT(m_lid != (PL_ListenerId)-1);
 	GR_Graphics * pG = getGraphics();
 	formatAll(); // Do we keep this or not?
+	m_bFinishedInitialCheck = false;
+	m_iPrevPos = 0;
+	m_iGrammarCount = 0;
 	if(m_pView)
 	{
 		m_pView->setLayoutIsFilling(false);
@@ -591,6 +625,14 @@ void FL_DocLayout::setView(FV_View* pView)
 			{
 				addBackgroundCheckReason(bgcrDebugFlash);
 			}
+			m_pPrefs->getPrefsValueBool(static_cast<const XML_Char *>("AutoGrammarCheck"),&b);
+			if (b)
+			{
+				addBackgroundCheckReason(bgcrGrammar);
+				m_bAutoGrammarCheck = true;
+				m_iGrammarCount = 0;
+				m_iPrevPos = 0;
+			}
 		}
 	}
 }
@@ -819,7 +861,7 @@ void FL_DocLayout::removeEndnoteContainer(fp_EndnoteContainer * pECon)
 	{
 		pECon->getNext()->setPrev(pECon->getPrev());
 	}
-	fl_EndnoteLayout * pEL = static_cast<fl_EndnoteLayout *>(pECon->getSectionLayout());
+//	fl_EndnoteLayout * pEL = static_cast<fl_EndnoteLayout *>(pECon->getSectionLayout());
 //	pDSL = static_cast<fl_DocSectionLayout *>(pEL->myContainingLayout());
 //	if(!pDSL->isCollapsing())
 	{
@@ -1637,6 +1679,11 @@ fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos) const
 		{
 		case PTX_Block:
 			pBL = static_cast<fl_BlockLayout *>(pL);
+			while(pBL && !pBL->canContainPoint())
+			{
+				pBL = pBL->getPrevBlockInDocument();
+			}
+				  
 			break;
 
 		case PTX_Section:
@@ -1652,7 +1699,17 @@ fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos) const
 		return NULL;
 	}
 
-	if(pBL->getSectionLayout()->getType() == FL_SECTION_HDRFTR)
+	UT_return_val_if_fail( pBL, NULL );
+	
+	fl_ContainerLayout * pMyC = pBL->myContainingLayout();
+	while(pMyC && (pMyC->getContainerType() != FL_CONTAINER_DOCSECTION)
+	      && (pMyC->getContainerType() != FL_CONTAINER_HDRFTR)
+	      && (pMyC->getContainerType() != FL_CONTAINER_SHADOW))
+	{
+	  pMyC = pMyC->myContainingLayout();
+	}
+	if((pMyC->getContainerType() == FL_CONTAINER_HDRFTR)
+	      || (pMyC->getContainerType() == FL_CONTAINER_SHADOW))
 	{
 		fl_HdrFtrShadow * pShadow = NULL;
 		FV_View * pView = getView();
@@ -1665,8 +1722,21 @@ fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos) const
 //
 			if(!pShadow->getHdrFtrSectionLayout()->isPointInHere(pos))
 			{
-				fl_HdrFtrSectionLayout * pHF = static_cast<fl_HdrFtrSectionLayout *>(pBL->getSectionLayout());
-				if(pHF->isPointInHere(pos))
+			        fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pBL->getSectionLayout());
+				while(pCL && pCL->getContainerType() != FL_CONTAINER_HDRFTR && pCL->getContainerType() != FL_CONTAINER_DOCSECTION)
+				{
+				  if(pCL == pCL->myContainingLayout())
+				  {
+				    break;
+				  }
+				  pCL = pCL->myContainingLayout();
+				}
+				fl_HdrFtrSectionLayout * pHF = NULL;
+				if(pCL && pCL->getContainerType() == FL_CONTAINER_HDRFTR)
+				{ 
+				     pHF = static_cast<fl_HdrFtrSectionLayout *>(pCL);
+				}
+				if(pHF && pHF->isPointInHere(pos))
 				{
 					pShadow = pHF->getFirstShadow();
 					if(pShadow)
@@ -1683,20 +1753,26 @@ fl_BlockLayout* FL_DocLayout::findBlockAtPosition(PT_DocPosition pos) const
 				}
 				// Ok, we're really confused now, point is nowhere to be found.
 				// It might be OK if pos-1 is in here, though...
-				if (!pShadow->getHdrFtrSectionLayout()->isPointInHere(pos-1))
+				if (pShadow && !pShadow->getHdrFtrSectionLayout()->isPointInHere(pos-1))
 				{
 					//			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 				}
 			}
 
 		}
+		else if(pMyC->getContainerType() == FL_CONTAINER_SHADOW)
+		{
+		        pShadow = static_cast<fl_HdrFtrShadow *>(pMyC);
+		}
 		else
 		{
-			pShadow = static_cast<fl_HdrFtrSectionLayout *>(pBL->getSectionLayout())->getFirstShadow();
+			pShadow = static_cast<fl_HdrFtrSectionLayout *>(pMyC)->getFirstShadow();
 		}
 		fl_BlockLayout * ppBL = NULL;
 		if(pShadow != NULL)
+		{
 			ppBL = static_cast<fl_BlockLayout *>(pShadow->findMatchingContainer(pBL));
+		}
 		else
 		{
 			UT_DEBUGMSG(("No Shadow! But there should be ! \n"));
@@ -1968,7 +2044,7 @@ void FL_DocLayout::updateLayout()
 	fl_SectionLayout* pSL = m_pFirstSection;
 	while (pSL)
 	{
-		pSL->updateLayout();
+		pSL->updateLayout(false);
 		if(pSL->getType() == FL_SECTION_DOC)
 		{
 			if(static_cast<fl_DocSectionLayout *>(pSL)->needsRebuild())
@@ -2100,7 +2176,7 @@ FL_DocLayout::_toggleAutoSpell(bool bSpell)
 				if(b->getContainerType() == FL_CONTAINER_BLOCK)
 				{
 					static_cast<fl_BlockLayout *>(b)->removeBackgroundCheckReason(bgcrSpelling);
-					static_cast<fl_BlockLayout *>(b)->getSquiggles()->deleteAll();
+					static_cast<fl_BlockLayout *>(b)->getSpellSquiggles()->deleteAll();
 					b = static_cast<fl_BlockLayout *>(b)->getNextBlockInDocument();
 				}
 				else
@@ -2123,6 +2199,91 @@ FL_DocLayout::_toggleAutoSpell(bool bSpell)
 	}
 }
 
+
+/*!
+ Toggle auto spell-checking state
+ \param bGrammar True if grammar-checking should be enabled, false otherwise
+ When disabling grammar checking, all squiggles are deleted.
+ When enabling grammar, force a full check of the document.
+*/
+void
+FL_DocLayout::_toggleAutoGrammar(bool bGrammar)
+{
+	bool bOldAutoGrammar = getAutoGrammarCheck();
+	UT_DEBUGMSG(("_toggleAutoGrammar %d \n",bGrammar));
+	// Add reason to background checker
+	if (bGrammar)
+	{
+		UT_DEBUGMSG(("Adding Auto GrammarCheck  \n"));
+		addBackgroundCheckReason(bgcrGrammar);
+		m_bAutoGrammarCheck = true;
+	}
+	else
+	{
+		UT_DEBUGMSG(("Removing Auto Grammar  \n"));
+		removeBackgroundCheckReason(bgcrGrammar);
+		m_bAutoGrammarCheck = false;
+	}
+
+	xxx_UT_DEBUGMSG(("FL_DocLayout::_toggleAutoGrammar (%s)\n",
+					 bGrammar ? "true" : "false" ));
+
+	if (bGrammar)
+	{
+		UT_DEBUGMSG(("Rechecking Grammar in blocks \n"));
+		// When enabling, recheck the whole document
+		fl_DocSectionLayout * pSL = getFirstSection();
+		if(pSL)
+		{
+			fl_ContainerLayout* b = pSL->getFirstLayout();
+			while (b)
+			{
+				// TODO: just check and remove matching squiggles
+				// for now, destructively recheck the whole thing
+				if(b->getContainerType() == FL_CONTAINER_BLOCK)
+				{
+					UT_DEBUGMSG(("Add block  %x for Grammar check \n",b));
+					queueBlockForBackgroundCheck(bgcrGrammar, static_cast<fl_BlockLayout *>(b));
+					b = static_cast<fl_BlockLayout *>(b)->getNextBlockInDocument();
+				}
+				else
+				{
+					b = b->getNext();
+				}
+			}
+		}
+	}
+	else
+	{
+		// Disabling, so remove the squiggles too
+		fl_DocSectionLayout * pSL = getFirstSection();
+		if(pSL)
+		{
+			fl_ContainerLayout* b = pSL->getFirstLayout();
+			while (b)
+			{
+				if(b->getContainerType() == FL_CONTAINER_BLOCK)
+				{
+					static_cast<fl_BlockLayout *>(b)->removeBackgroundCheckReason(bgcrGrammar);
+					static_cast<fl_BlockLayout *>(b)->getGrammarSquiggles()->deleteAll();
+					b = static_cast<fl_BlockLayout *>(b)->getNextBlockInDocument();
+				}
+				else
+				{
+					b = b->getNext();
+				}
+			}
+		}
+		if (bOldAutoGrammar)
+		{
+			// If we're here, it was set to TRUE before but now it is
+			// being set to FALSE. This means that it is the user
+			// setting it. That's good.
+			m_pView->draw(NULL);
+		}
+	}
+}
+
 void FL_DocLayout::_toggleAutoSmartQuotes(bool bSQ)
 {
 	setPendingSmartQuote(NULL, 0);  // avoid surprises
@@ -2166,7 +2327,7 @@ FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
 //	{
 //		return;
 //	}
-
+	xxx_UT_DEBUGMSG(("BAckground check called. \n"));
 	// Don't spell check while printing!
 	if(pDocLayout->m_pG->queryProperties(GR_Graphics::DGP_PAPER))
 	{
@@ -2176,6 +2337,7 @@ FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
 	// Don't spell check if disabled, or already happening
 	if(pDocLayout->m_bStopSpellChecking || pDocLayout->m_bImSpellCheckingNow || pDocLayout->isLayoutFilling())
 	{
+	  xxx_UT_DEBUGMSG(("Already spellchecking!!!!!!!! \n"));
 		return;
 	}
 
@@ -2222,6 +2384,12 @@ FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
 					mask = (1 << bitdex);
 					if (pB->hasBackgroundCheckReason(mask))
 					{
+					        if(!pDocLayout->m_bFinishedInitialCheck && pDocLayout->m_iPrevPos > pB->getPosition())
+						{
+						     pDocLayout->m_bFinishedInitialCheck = true;
+						}
+						pDocLayout->m_iPrevPos = pB->getPosition();
+
 					// Note that we remove this reason from queue
 					// before checking it (otherwise asserts could
 					// trigger redundant recursive calls)
@@ -2244,6 +2412,30 @@ FL_DocLayout::_backgroundCheck(UT_Worker * pWorker)
 							}
 							break;
 						}
+						case bgcrGrammar:
+						{
+						        if(!pDocLayout->m_bFinishedInitialCheck)
+							{
+							      if(pDocLayout->m_iGrammarCount < 4)
+							      {
+								   pDocLayout->m_iGrammarCount++;
+								   pDocLayout->m_bImSpellCheckingNow = false;
+								   return;
+							      }
+							      pDocLayout->m_iGrammarCount = 0;
+							}
+
+							UT_DEBUGMSG(("Grammar checking block %x directly \n",pB));
+							XAP_App * pApp = pDocLayout->getView()->getApp();
+     //
+     // If a grammar checker plugin is loaded it will check the block now.
+     //
+							pApp->notifyListeners(pDocLayout->getView(),AV_CHG_BLOCKCHECK,reinterpret_cast<void *>(pB));
+							pB->removeBackgroundCheckReason(mask);
+							pB->drawGrammarSquiggles();
+							break;
+						}
+
 						case bgcrSmartQuotes:
 						default:
 							pB->removeBackgroundCheckReason(mask);
@@ -2295,8 +2487,12 @@ FL_DocLayout::queueBlockForBackgroundCheck(UT_uint32 iReason,
 	if (!m_pBackgroundCheckTimer)
 	{
 	    int inMode = UT_WorkerFactory::IDLE | UT_WorkerFactory::TIMER;
+	    if(getView() && getView()->isGrammarLoaded() && m_bAutoGrammarCheck)
+	    {
+	         inMode = UT_WorkerFactory::TIMER;
+	    }
 	    UT_WorkerFactory::ConstructMode outMode = UT_WorkerFactory::NONE;
-
+	    
 	    m_pBackgroundCheckTimer = UT_WorkerFactory::static_constructor (_backgroundCheck, this, inMode, outMode, m_pG);
 
 	    UT_ASSERT(m_pBackgroundCheckTimer);
@@ -2356,6 +2552,9 @@ void FL_DocLayout::dequeueAll(void)
 	{
 		m_vecUncheckedBlocks.deleteNthItem(i);	
 	}
+	UT_DEBUGMSG(("Dequeue all \n"));
+
+	m_PendingBlockForGrammar = NULL;
 	m_bStopSpellChecking = true;
 	if(m_pBackgroundCheckTimer)
 	{
@@ -2368,6 +2567,40 @@ void FL_DocLayout::dequeueAll(void)
 	}
 }
 
+/*!
+ * Set the next block to be grammar checked. It won't actually get checked
+ * until the insertPoint leaves this block.
+ */
+void FL_DocLayout::setPendingBlockForGrammar(fl_BlockLayout * pBL)
+{
+  xxx_UT_DEBUGMSG(("Pending called with block %x pending %x \n",pBL,m_PendingBlockForGrammar));
+  if(!m_bAutoGrammarCheck)
+    return;
+  if((m_PendingBlockForGrammar != NULL) && (m_PendingBlockForGrammar != pBL))
+    {
+      xxx_UT_DEBUGMSG(("Block %x queued \n",m_PendingBlockForGrammar));
+      queueBlockForBackgroundCheck(bgcrGrammar,m_PendingBlockForGrammar,true);
+    }
+  m_PendingBlockForGrammar = pBL;
+}
+
+
+/*!
+ * This is called from fv_View::_fixPointCoords to actually queue a grammar 
+ * check a pending block.
+ */
+void FL_DocLayout::triggerPendingBlock(fl_BlockLayout * pBL)
+{
+  xxx_UT_DEBUGMSG(("Trigger called with block %x pending %x \n",pBL,m_PendingBlockForGrammar));
+  if(!m_bAutoGrammarCheck)
+    return;
+  if((m_PendingBlockForGrammar != NULL) && (m_PendingBlockForGrammar != pBL))
+    {
+      queueBlockForBackgroundCheck(bgcrGrammar,m_PendingBlockForGrammar,true);
+      m_PendingBlockForGrammar = NULL;
+     }
+}
+
 /*!
  Remove block from background checking queue
  \param pBlock Block to remove
@@ -2388,7 +2621,11 @@ FL_DocLayout::dequeueBlockForBackgroundCheck(fl_BlockLayout *pBlock)
 		m_vecUncheckedBlocks.deleteNthItem(i);
 		bRes = true;
 	}
-
+	if(pBlock == m_PendingBlockForGrammar)
+	  {
+	    xxx_UT_DEBUGMSG(("Dequeue block %x in dequeue \n",pBlock));
+	    m_PendingBlockForGrammar = NULL;
+	  }
 	// When queue is empty, kill timer
 	if (m_vecUncheckedBlocks.getItemCount() == 0)
 	{
@@ -2785,6 +3022,15 @@ fl_DocSectionLayout* FL_DocLayout::findSectionForHdrFtr(const char* pszHdrFtrID)
 		pDocLayout->_toggleAutoSpell( b );
 	}
 
+	// grammar check
+	pPrefs->getPrefsValueBool(static_cast<const XML_Char *>(AP_PREF_KEY_AutoGrammarCheck), &b );
+	changed = changed || (b != pDocLayout->m_bAutoSpellCheck);
+	if(b != pDocLayout->m_bAutoGrammarCheck || (pDocLayout->m_iGraphicTick < 2))
+	{
+		pDocLayout->m_bAutoGrammarCheck = b;
+		pDocLayout->_toggleAutoGrammar( b );
+	}
+
 // autosave
 
 	UT_String stTmp;
