@@ -18,7 +18,7 @@
 
 rm -rf autom4te.cache
 
-abi_version=2.3.1
+abi_version=2.3.5
 
 # abi_plugins is the list of subdirectories that are plugins
 # The definition of a plugin is a directory containing a file called "plugin.m4".
@@ -486,8 +486,25 @@ rm -f $abi_topdir/acinclude.m4
 cat >> configure.ac << EOF
 AC_CONFIG_FILES([
 $abi_makefiles
+])
+AC_CONFIG_FILES([
 EOF
-($CONFIG_SHELL $abi_topdir/nextgen.sh --query --list-files) >> configure.ac
+($CONFIG_SHELL $abi_topdir/nextgen.sh --query --list-files) | grep "^tools/" >> configure.ac
+cat >> configure.ac << EOF
+])
+AC_CONFIG_FILES([
+EOF
+($CONFIG_SHELL $abi_topdir/nextgen.sh --query --list-files) | grep "^wp/impexp/" | grep -v "/graphics/" >> configure.ac
+cat >> configure.ac << EOF
+])
+AC_CONFIG_FILES([
+EOF
+($CONFIG_SHELL $abi_topdir/nextgen.sh --query --list-files) | grep "^wp/impexp/" | grep "/graphics/" >> configure.ac
+cat >> configure.ac << EOF
+])
+AC_CONFIG_FILES([
+EOF
+($CONFIG_SHELL $abi_topdir/nextgen.sh --query --list-files) | grep -v "^wp/impexp/" | grep -v "^tools/" >> configure.ac
 cat >> configure.ac << EOF
 ])
 AC_OUTPUT
